package com.yizhi.site.application.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.site.application.domain.DataClassification;
import com.yizhi.site.application.domain.EventTrack;
import com.yizhi.site.application.mapper.EventTrackMapper;
import com.yizhi.site.application.service.DataClassificationService;
import com.yizhi.site.application.service.EventTrackService;
import com.yizhi.site.application.vo.domain.EventTrackVo;
import com.yizhi.site.application.vo.site.DayNumVo;
import com.yizhi.site.application.vo.site.EventTrackDayNumVo;
import com.yizhi.site.application.vo.site.EventTrackDeptNumVo;
import com.yizhi.site.application.vo.site.EventTrackTypeNumVo;
import com.yizhi.system.application.feign.DictionaryFeignClients;
import com.yizhi.system.application.vo.domain.Dictionary;
import com.yizhi.training.application.feign.LeaveWordClient;
import com.yizhi.training.application.vo.domain.AuthzUserGroupVo;
import com.yizhi.util.application.date.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * com.yizhi.site.application.service.impl
 *
 * @author yanglang
 * @create 2025-01-14 10:15:57
 */
@Service
public class EventTrackServiceImpl extends ServiceImpl<EventTrackMapper, EventTrack> implements EventTrackService {
    private static final Logger logger= LoggerFactory.getLogger(EventTrackServiceImpl.class);

    @Autowired
    private DictionaryFeignClients dictionaryFeignClients;

    @Autowired
    private LeaveWordClient leaveWordClient;

    @Autowired
    private DataClassificationService dataClassificationService;

    @Autowired
    IdGenerator idGenerator;

    @Override
    public void addEvent(String type,Long businessId){
        logger.info("新增埋点{},{}",type,businessId);
        RequestContext context = ContextHolder.get();
        Long userId = context.getAccountId();
        if(Objects.equals(type,"event_login")){
            userId = businessId;
            ContextHolder.get().setAccountId(userId);
        }
        EventTrack eventTrack = new EventTrack();
        if(businessId!=null){
            DataClassification dataClassification = dataClassificationService.selectById(businessId);
            if(dataClassification!=null){
                eventTrack.setRemark1(dataClassification.getName());
            }
        }
        Date toDay = new Date();
        eventTrack.setBusinessId(businessId);
        eventTrack.setCreateById(userId);
        eventTrack.setCreateByName(context.getAccountName());
        eventTrack.setCreateTime(toDay);
        eventTrack.setUpdateById(userId);
        eventTrack.setUpdateByName(context.getAccountName());
        eventTrack.setUpdateTime(toDay);
        AuthzUserGroupVo authzUserGroupVo = leaveWordClient.getAuthzUserGroup(userId);
        if(authzUserGroupVo!=null){
            eventTrack.setDeptId(authzUserGroupVo.getId());
            eventTrack.setDeptName(authzUserGroupVo.getName());
        }
        String key = "event_track_type";
        List<Dictionary> dictionaryList = dictionaryFeignClients.listChildDictionary(key,false,1);
        if(CollUtil.isNotEmpty(dictionaryList)){
            Optional<Dictionary> optional = dictionaryList.stream().filter(d-> Objects.equals(d.getCode(),type)).findFirst();
            if(optional!=null&&optional.isPresent()){
                Dictionary dictionary = optional.get();
                eventTrack.setSubType(dictionary.getCnName());
                eventTrack.setType(Integer.valueOf(dictionary.getValue()));
            }
        }
        if(Objects.equals(type,"event_login")){
            EntityWrapper<EventTrack> wrapper = new EntityWrapper<EventTrack>();
            wrapper.eq("business_id",userId);
            wrapper.eq("type",eventTrack.getType());
            wrapper.between("create_time", DateUtil.parse(DateUtil.toDay(toDay),"yyyy-MM-dd"),toDay);
            EventTrack track = this.selectOne(wrapper);
            if(track!=null&&track.getId()!=null){
                return;
            }
        }
        eventTrack.setId(idGenerator.generate());
        this.insert(eventTrack);
    }

    @Override
    public EventTrackDayNumVo getTodayNum(String date) {
        if(ObjectUtils.isEmpty(date)){
            date = DateUtil.toDay(new Date());
        }
        return this.baseMapper.getTodayNum(date);
    }


    @Override
    public Integer getNum(String typeStr,Long businessId){
        String key = "event_track_type";
        Integer type = 0;
        List<Dictionary> dictionaryList = dictionaryFeignClients.listChildDictionary(key,false,1);
        if(CollUtil.isNotEmpty(dictionaryList)){
            Optional<Dictionary> optional = dictionaryList.stream().filter(d-> Objects.equals(d.getCode(),typeStr)).findFirst();
            if(optional!=null&&optional.isPresent()){
                Dictionary dictionary = optional.get();
                type =Integer.valueOf(dictionary.getValue());
            }
        }
        if(type==0){
            return 0;
        }

        return this.baseMapper.getNum(type,businessId);
    }

    @Override
    public DayNumVo getNumByType(String date,List<Integer> types) {
        if(ObjectUtils.isEmpty(date)){
            date = "3";
        }
        String dateString =getDateString(date);
        return this.baseMapper.getNumByType(date,dateString,types);
    }

    @Override
    public List<EventTrackTypeNumVo> queryNumByType(String date,List<Integer> types,String subName) {
        if(ObjectUtils.isEmpty(date)){
            date = "3";
        }
        String dateString =getDateString(date);
        Long id = null;
        if(!ObjectUtils.isEmpty(subName)) {
            String key = "event_track_type";
            List<Dictionary> dictionaryList = dictionaryFeignClients.listChildDictionary(key, false, 1);
            if (CollUtil.isNotEmpty(dictionaryList)) {
                Optional<Dictionary> optional = dictionaryList.stream().filter(d -> Objects.equals(d.getValue(), "6")&&Objects.equals(d.getCnName(), subName)).findFirst();
                if (optional != null && optional.isPresent()) {
                    Dictionary dictionary = optional.get();
                    id = Long.valueOf(dictionary.getCode());
                }
            }
        }
        List<String> subNameList = null;
        if(id!=null) {
            List<DataClassification>  classificationList =  dataClassificationService.getSon(id);
            if(CollectionUtil.isNotEmpty(classificationList)){
                subNameList = classificationList.stream().map(DataClassification::getName).collect(Collectors.toList());
            }
        }
        List<EventTrackTypeNumVo> eventTrackTypeNumVoList = this.baseMapper.queryNumByType(null,date,dateString,types,subNameList);
        if(CollectionUtil.isNotEmpty(subNameList)){
            subNameList.stream().forEach(name->{
                EventTrackTypeNumVo eventTrackTypeNumVo = new EventTrackTypeNumVo();
                eventTrackTypeNumVo.setNum(0);
                eventTrackTypeNumVo.setSubType(name);
                if(CollectionUtil.isNotEmpty(eventTrackTypeNumVoList)){
                   Optional optional = eventTrackTypeNumVoList.stream().filter(e->Objects.equals(e.getSubType(),name)).findFirst();
                    if(optional==null||!optional.isPresent()){
                        eventTrackTypeNumVoList.add(eventTrackTypeNumVo);
                    }
                }else{
                    eventTrackTypeNumVoList.add(eventTrackTypeNumVo);
                }
            });
        }

        return eventTrackTypeNumVoList;
    }

    @Override
    public List<EventTrackDeptNumVo> queryDeptByType(String date, List<Integer> types) {
        if(ObjectUtils.isEmpty(date)){
            date = "3";
        }
        String dateString =getDateString(date);

        List<EventTrackDeptNumVo> eventTrackDeptNumVoList = this.baseMapper.queryGropByList();
        if(CollUtil.isNotEmpty(eventTrackDeptNumVoList)){
            String finalDate = date;
            eventTrackDeptNumVoList.stream().forEach(eventTrackDeptNumVo -> {
                List<EventTrackTypeNumVo> eventTrackTypeNumVoList = this.baseMapper.queryNumByType(eventTrackDeptNumVo.getDeptId(),finalDate, dateString,types,null);
                if(CollectionUtil.isNotEmpty(eventTrackTypeNumVoList)){
                    List<EventTrackTypeNumVo> voList = Lists.newArrayList();
                    eventTrackTypeNumVoList.stream().forEach(numVo->{
                       Optional<EventTrackTypeNumVo> optional= voList.stream().filter(v->Objects.equals(v.getType(),numVo.getType())).findFirst();
                       if(optional!=null&&optional.isPresent()){
                           EventTrackTypeNumVo vo = optional.get();
                           if(numVo.getNum()!=null) {
                               if(vo.getNum()!=null) {
                                   vo.setNum(numVo.getNum()+vo.getNum());
                               }else{
                                   vo.setNum(numVo.getNum());
                               }
                           }
                       }else{
                           if(numVo.getType()==13){
                               numVo.setSubType("评论总书");
                           }
                           voList.add(numVo);
                       }
                    });
                    eventTrackDeptNumVo.setEventTrackTypeNumVoList(voList);
                }else {
                    eventTrackDeptNumVo.setEventTrackTypeNumVoList(eventTrackTypeNumVoList);
                }

                types.stream().forEach(t->{
                    EventTrackTypeNumVo vo = new EventTrackTypeNumVo();
                    vo.setType(t);
                    vo.setNum(0);
                    if(CollectionUtil.isNotEmpty(eventTrackDeptNumVo.getEventTrackTypeNumVoList())) {
                        Optional<EventTrackTypeNumVo> optional = eventTrackDeptNumVo.getEventTrackTypeNumVoList().stream().filter(v -> Objects.equals(v.getType(), t)).findFirst();
                        if(!(optional!=null&&optional.isPresent())){
                            eventTrackDeptNumVo.getEventTrackTypeNumVoList().add(vo);
                        }
                    }else{
                        List<EventTrackTypeNumVo> voList = Lists.newArrayList();
                        voList.add(vo);
                        eventTrackDeptNumVo.setEventTrackTypeNumVoList(voList);
                    }
                });

            });
        }
        return eventTrackDeptNumVoList;
    }

    @Override
    public List<EventTrackVo> queryListByType(Integer type, Long accountId) {
        EventTrack eventTrack = new EventTrack();
        eventTrack.setType(type);
        eventTrack.setCreateById(accountId);
        EntityWrapper entityWrapper = new EntityWrapper<>(eventTrack);
        List<String> orderList = Lists.newArrayList();
        orderList.add("create_time");
        entityWrapper.orderDesc(orderList);
        List<EventTrack> eventTrackList = this.selectList(entityWrapper);
        List<EventTrackVo> eventTrackVoList =Lists.newArrayList();
        if(CollectionUtil.isNotEmpty(eventTrackList)){
            eventTrackList.stream().forEach(e->{
                EventTrackVo vo = new EventTrackVo();
                BeanUtils.copyProperties(e,vo);
                eventTrackVoList.add(vo);
            });
        }
        return eventTrackVoList;
    }

    public String getDateString(String date) {
        String dateString = DateUtil.toDay(new Date());
        String[] dateArrays = dateString.split("-");
        if(Objects.equals(date,"1")){
            dateString = dateArrays[0];
        }
        if(Objects.equals(date,"2")){
            dateString = dateArrays[0]+"-"+dateArrays[1];
        }
        if(Objects.equals(date,"4")){
            Calendar calendar = Calendar.getInstance();
            int week = calendar.get(Calendar.WEEK_OF_YEAR);
            if(week==1){
                dateString = String.valueOf(Integer.valueOf(dateArrays[0])-1);
            }else{
                dateString = dateArrays[0];
            }
        }
        return dateString;
    }
}
